@@ -111,6 +111,11 @@ class Exchange:
         """The vertices in this exchange."""
         return self._vertices
 
+    @property
+    def chain(self) -> bool:
+        """Returns True if this exchange represents a chain."""
+        return self._vertices[0].isNdd()
+
     def __len__(self):
         """Returns the number of vertices (donors) in the exchange. Note that
         in the case of a non-directed altruistic donor, this will be greater
@@ -141,6 +146,62 @@ class Exchange:
         """Get all of the recipients."""
         return [v.donor.recipient for v in self._vertices if not v.isNdd()]
 
+    def backarc_exchanges_uk(self) -> list[Exchange]:
+        """Return the exchanges that contain backarcs from this exchange (by
+        the UK definition of backarc).
+
+        A backarc, by UK counting, is only defined for exchanges with three
+        transplants (i.e., three donors). Let the three donors be A, B, and C.
+        A backarc exists if A (respectively B and C) can donate to the paired
+        recipient of C (respectively A and B).
+
+        :return: A list containing each Exchange object that has a backarc from
+            this Exchange.
+        """
+        if len(self) == 2:
+            return []
+        if len(self) != 3:
+            raise Exception(
+                "Backarcs are not defined for exchanges of length 4 or more"
+            )
+        backarc_exchanges = []
+        for ind, v in enumerate(self):
+            prev_v = self[(ind - 1) % len(self)]
+            prev_donor = prev_v.donor
+            donor = v.donor
+            # Note that a backarc may exist in an alternate (rather than
+            # embedded) exchange if this exchange is a chain, as an exchange is
+            # an alternate if it has the same set of recipients (unlike
+            # embedded exchanges, which have a strict subset only). This can
+            # occur in a long chain, if there is also a two-way cycle between
+            # the two vertices representing directed (paired) donors.
+            if self.chain:
+                for exchange in self.alternates:
+                    if len(exchange) != 2:
+                        continue
+                    donors = [v.donor for v in exchange]
+                    if donor in donors and prev_donor in donors:
+                        backarc_exchanges.append(exchange)
+            # By definition, an embedded exchange has length two here.
+            for exchange in self.embedded:
+                donors = [v.donor for v in exchange]
+                if donor in donors and prev_donor in donors:
+                    backarc_exchanges.append(exchange)
+        return backarc_exchanges
+
+    def num_backarcs_uk(self) -> int:
+        """Return the number of backarcs in this exchange (by the UK definition
+        of backarc).
+
+        A backarc, by UK counting, is only defined for exchanges with three
+        transplants (i.e., three donors). Let the three donors be A, B, and C.
+        A backarc exists if A (respectively B and C) can donate to the paired
+        recipient of C (respectively A and B).
+
+        :return: The number of backarcs in this Exchange.
+        """
+        return len(self.backarc_exchanges_uk())
+
     @property
     def alternates(self) -> list[Exchange]:
         """Return the alternate exchanges for this exchange. An alternate
@@ -489,7 +550,10 @@ def build_alternates_and_embeds(exchanges: list[Exchange]) -> None:
     are paired with multiple donors) or perhaps different donors and recipients
     are selected for transplant.  An embedded exchange is one that matches some
     (but not all) of the recipients within this exchange, but still does not
-    match any recipients not in this exchange.
+    match any recipients not in this exchange. Note that in both cases, if the
+    exchange is a chain, we require that any alternate or embedded exchanges
+    that are also chains must still be initiated by the same non-directed
+    donor.
 
     :param exchanges: The exchanges to search for alternates
     """
@@ -505,11 +569,29 @@ def build_alternates_and_embeds(exchanges: list[Exchange]) -> None:
         recipients = exchange.all_recipients()
         # Sort the recipients so we don't think [1,2] is different to [2,1]
         recipients.sort(key=lambda r: r.id)
-        exchange.add_alternates(cache[tuple(recipients)])
+        if exchange.chain:
+            first = exchange[0].donor
+            exchange.add_alternates(
+                e
+                for e in cache[tuple(recipients)]
+                if not e.chain or e[0].donor == first
+            )
+        else:
+            exchange.add_alternates(e for e in cache[tuple(recipients)] if not e.chain)
         # Note: We allow a chain consisting of only a non-directed donor, but
         # they should not be counted as an "embedded" exchange, hence minimum
         # size of 1 below
         for embed_size in range(1, len(recipients)):
             for subset in combinations(recipients, embed_size):
                 # Recipients is already sorted, so subsets should be too
-                exchange.add_embeddeds(cache[tuple(subset)])
+                if exchange.chain:
+                    first = exchange[0].donor
+                    exchange.add_embeddeds(
+                        e
+                        for e in cache[tuple(subset)]
+                        if not e.chain or e[0].donor == first
+                    )
+                else:
+                    exchange.add_embeddeds(
+                        e for e in cache[tuple(subset)] if not e.chain
+                    )
