@@ -129,7 +129,7 @@ class EffectiveTwoWay(Objective):
             return 0
         if len(exchange) == 2:
             return 1
-        if len(exchange) == 3 and len(exchange.embedded) >= 1:
+        if len(exchange) == 3 and exchange.num_backarcs_uk() >= 1:
             return 1
         return 0
 
@@ -163,9 +163,7 @@ class BackArcs(Objective):
         :return: The number of backarcs in the exchange
         """
         if len(exchange) == 3:
-            if exchange[0].isNdd():
-                return 1 + len(exchange.embedded)
-            return len(exchange.embedded)
+            return exchange.num_backarcs_uk()
         return 0
 
     def describe(self) -> str:
