@@ -105,6 +105,48 @@ def test_instance_two_embedded(test_two):
             assert len(cycle.embedded) == 0
 
 
+@pytest.fixture(
+    params=[
+        ("JSON", fileio.read_json("tests/test_instances/test3b.json")),
+        ("XML", fileio.read_xml("tests/test_instances/test3b.xml")),
+    ]
+)
+def test_threeb(request):
+    yield request.param
+
+
+def test_instance_threeb_read(test_threeb):
+    instance = test_threeb[1]
+    graph = CompatibilityGraph(instance)
+    assert graph.size == 4
+    assert len(graph.edges()) == len(instance.transplants())
+
+
+def test_instance_threeb_cycles(test_threeb):
+    instance = test_threeb[1]
+    graph = CompatibilityGraph(instance)
+
+    cycles = graph.findCycles(3)
+    build_alternates_and_embeds(cycles)
+    assert len(cycles) == 7
+    cycle_indices = [[int(v.donor.id) for v in cycle] for cycle in cycles]
+    assert [1, 2] in cycle_indices
+    assert [1, 2, 3] in cycle_indices
+    assert [1, 3] in cycle_indices
+    assert [1, 3, 4] in cycle_indices
+    assert [1, 4] in cycle_indices
+    assert [1, 4, 3] in cycle_indices
+    assert [3, 4] in cycle_indices
+
+    cycles = graph.findCycles(2)
+    assert len(cycles) == 4
+    cycle_indices = [[int(v.donor.id) for v in cycle] for cycle in cycles]
+    assert [1, 2] in cycle_indices
+    assert [1, 3] in cycle_indices
+    assert [1, 4] in cycle_indices
+    assert [3, 4] in cycle_indices
+
+
 @pytest.fixture(
     params=[
         ("JSON", fileio.read_json("tests/test_instances/test5.json")),
