@@ -33,6 +33,13 @@ def test5_graph():
     return graph
 
 
+@pytest.fixture(scope="module")
+def test7_graph():
+    instance = fileio.read_json("tests/test_instances/test7.json")
+    graph = graphing.CompatibilityGraph(instance)
+    return graph
+
+
 def test_transplant_count_test1(test1_graph):
     obj = model.TransplantCount()
     cycles = test1_graph.findCycles(3)
@@ -73,11 +80,11 @@ def test_effective_twoway_count_test4(test4_graph):
     for cycle in cycles:
         indices = [int(v.donor.id) for v in cycle]
         if indices == [1, 2]:
-            assert obj.value(test1_graph, cycle) == 1
+            assert obj.value(test4_graph, cycle) == 1
         if indices == [1, 2, 4]:
-            assert obj.value(test1_graph, cycle) == 1
+            assert obj.value(test4_graph, cycle) == 1
         if indices == [2, 4, 3]:
-            assert obj.value(test1_graph, cycle) == 0
+            assert obj.value(test4_graph, cycle) == 0
 
 
 def test_backarcs_test3b(test3b_graph):
@@ -87,74 +94,129 @@ def test_backarcs_test3b(test3b_graph):
     for cycle in cycles:
         indices = [int(v.donor.id) for v in cycle]
         if indices == [1, 2]:
-            assert obj.value(test1_graph, cycle) == 0
+            assert obj.value(test3b_graph, cycle) == 0
         if indices == [1, 2, 3]:
-            assert obj.value(test1_graph, cycle) == 2
+            assert obj.value(test3b_graph, cycle) == 2
         if indices == [1, 3]:
-            assert obj.value(test1_graph, cycle) == 0
+            assert obj.value(test3b_graph, cycle) == 0
         if indices == [1, 3, 4]:
-            assert obj.value(test1_graph, cycle) == 3
+            assert obj.value(test3b_graph, cycle) == 3
         if indices == [1, 4]:
-            assert obj.value(test1_graph, cycle) == 0
+            assert obj.value(test3b_graph, cycle) == 0
         if indices == [1, 4, 3]:
-            assert obj.value(test1_graph, cycle) == 3
+            assert obj.value(test3b_graph, cycle) == 3
         if indices == [3, 4]:
-            assert obj.value(test1_graph, cycle) == 0
+            assert obj.value(test3b_graph, cycle) == 0
 
 
 def test_backarcs_test5(test5_graph):
     obj = model.BackArcs()
     chains = test5_graph.findChains(3)
-    graphing.build_alternates_and_embeds(chains)
+    cycles = test5_graph.findCycles(3)
+    graphing.build_alternates_and_embeds(chains + cycles)
+    checked = 0
     for chain in chains:
         indices = [int(v.donor.id) for v in chain]
         if indices == [1, 2]:
-            assert obj.value(test1_graph, chain) == 0
+            assert obj.value(test5_graph, chain) == 0
+            checked += 1
         if indices == [1, 3, 4]:
-            assert obj.value(test1_graph, chain) == 2
+            assert obj.value(test5_graph, chain) == 2
+            checked += 1
+    assert checked == 2
+
+
+def test_backarcs_test7(test7_graph):
+    obj = model.BackArcs()
+    chains = test7_graph.findChains(3)
+    cycles = test7_graph.findCycles(3)
+    graphing.build_alternates_and_embeds(chains + cycles)
+    checked = 0
+    for exchange in chains + cycles:
+        indices = [int(v.donor.id) for v in exchange]
+        if indices == [125, 24, 11]:
+            assert obj.value(test7_graph, exchange) == 2
+            checked += 1
+        if indices == [122, 83, 54]:
+            assert obj.value(test7_graph, exchange) == 2
+            checked += 1
+        if indices == [27, 60, 46]:
+            assert obj.value(test7_graph, exchange) == 1
+            checked += 1
+        if indices == [3, 102, 45]:
+            assert obj.value(test7_graph, exchange) == 1
+            checked += 1
+        if indices == [123, 104, 43]:
+            assert obj.value(test7_graph, exchange) == 2
+            checked += 1
+        if indices == [16, 44, 20]:
+            assert obj.value(test7_graph, exchange) == 1
+            checked += 1
+        if indices == [121, 87, 34]:
+            assert obj.value(test7_graph, exchange) == 2
+            checked += 1
+    assert checked == 7
 
 
 def test_threeway_count_test1(test1_graph):
     obj = model.ThreeWay()
     cycles = test1_graph.findCycles(3)
+    checked = 0
     for cycle in cycles:
         indices = [int(v.donor.id) for v in cycle]
         if indices == [1, 2]:
             assert obj.value(test1_graph, cycle) == 0
-        if indices == [1, 2, 3]:
+            checked += 1
+        if indices == [2, 3, 4]:
             assert obj.value(test1_graph, cycle) == 1
+            checked += 1
+    assert checked == 2
 
 
 def test_threeway_count_test3b(test3b_graph):
     obj = model.ThreeWay()
     cycles = test3b_graph.findCycles(3)
     graphing.build_alternates_and_embeds(cycles)
+    checked = 0
     for cycle in cycles:
         indices = [int(v.donor.id) for v in cycle]
         if indices == [1, 2]:
-            assert obj.value(test1_graph, cycle) == 0
+            assert obj.value(test3b_graph, cycle) == 0
+            checked += 1
         if indices == [1, 2, 3]:
-            assert obj.value(test1_graph, cycle) == 1
+            assert obj.value(test3b_graph, cycle) == 1
+            checked += 1
         if indices == [1, 3]:
-            assert obj.value(test1_graph, cycle) == 0
+            assert obj.value(test3b_graph, cycle) == 0
+            checked += 1
         if indices == [1, 3, 4]:
-            assert obj.value(test1_graph, cycle) == 1
+            assert obj.value(test3b_graph, cycle) == 1
+            checked += 1
         if indices == [1, 4]:
-            assert obj.value(test1_graph, cycle) == 0
+            assert obj.value(test3b_graph, cycle) == 0
+            checked += 1
         if indices == [1, 4, 3]:
-            assert obj.value(test1_graph, cycle) == 1
+            assert obj.value(test3b_graph, cycle) == 1
+            checked += 1
         if indices == [3, 4]:
-            assert obj.value(test1_graph, cycle) == 0
+            assert obj.value(test3b_graph, cycle) == 0
+            checked += 1
+    assert checked == 7
 
 
 def test_threeway_count_test5(test5_graph):
     obj = model.ThreeWay()
     chains = test5_graph.findChains(3)
+    checked = 0
     for chain in chains:
         indices = [int(v.donor.id) for v in chain]
         if indices == [1, 2]:
             assert obj.value(test5_graph, chain) == 0
+            checked += 1
         if indices == [1, 3]:
             assert obj.value(test5_graph, chain) == 0
+            checked += 1
         if indices == [1, 3, 4]:
             assert obj.value(test5_graph, chain) == 1
+            checked += 1
+    assert checked == 3
